# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name        :   BMVB.rb
# Description :   Draws a face that covers precisely current view
# Menu Item   :   Plugins\BringMyViewBack
# Author      :   Tomasz Marek
# Usage       :   Created face can be used for B&W image with alpha transparency
#	      :   taken form the same camera possition
#	      :   Face can be semitransparent and have some openings.. and so on
# Date        :   25/06/2008
# Type        :   Tool
# Version     :   1.00 - initial version
# Version     :   1.10 - easy replacing of a view component added
#-----------------------------------------------------------------------------

require 'sketchup.rb'

# -------------- Main prog ------------------------------------

class BMVB

# Main routine

def BMVB::frame

	model = Sketchup.active_model

		(BMVB.param_cam_view model.active_view)

end #method end

# ----------------------------------- About message
def BMVB::about
UI.messagebox("BMVB version 1.1. June 2008\nWritten by Tomasz Marek\n\nUSAGE: select an existing Scene or create new, run BMVB to create a new 'camera' component that will cover whole current view (can give wrong results with 2 point perspective & photomatched views). Can be used as a projection plane for overlaid rendering of the same view or for placing images into tutorials.\nIt is also helpful for finding actual position of a current camera in 3D model." , MB_MULTILINE , "Bring My View Back!") 
end

# -----------Extract the camera parameters of the current view ------------------------------------
def BMVB::param_cam_view( v )

h = v.vpheight
w = v.vpwidth
user_camera = v.camera
user_pers = user_camera.perspective?
user_eye = user_camera.eye

user_target = user_camera.target
user_dir = user_camera.direction
user_x = user_camera.xaxis 
user_y = user_camera.yaxis 
user_z = user_camera.zaxis
user_up = user_camera.up
user_ar = user_camera.aspect_ratio

if( user_pers == true ) 
	user_foc = user_camera.focal_length
	user_fov = user_camera.fov
end

model = Sketchup.active_model
comp = model.definitions.add "ViewFrame"

zero=Geom::Point3d.new 0,0,0

user_dir.length=2.0

x_dist = Geom::Vector3d.new(user_foc*w/h*Math.tan((user_fov/2).degrees),0,0)
y_dist = Geom::Vector3d.new(0,user_foc*Math.tan((user_fov/2).degrees),0)
z_dist = Geom::Vector3d.new 0,0,user_foc

comp.entities.add_line(zero, zero+z_dist)

pt1=zero+z_dist+y_dist-x_dist
pt2=zero+z_dist+y_dist+x_dist
pt3=zero+z_dist-y_dist+x_dist
pt4=zero+z_dist-y_dist-x_dist

face = comp.entities.add_face(pt1,pt2,pt3,pt4)

inst=model.entities.add_instance comp,user_eye

comp_dir=Geom::Transformation.new user_eye,user_dir
inst.transformation=comp_dir

end

end #Class end


# ---------------------------------------- Menu item
if( not file_loaded?("BMVB.rb") )

	main_menu = UI.menu("Plugins").add_submenu("[TM] BMVB")
	main_menu.add_item("Bring My View Back") { (BMVB.frame) }
	main_menu.add_separator
	main_menu.add_item("About BMVB") { (BMVB.about) }


end
file_loaded("BMVB.rb")